-------------------------------------------------------------------------------
-- Title      : pilote de souris PS/2
-- Project    : 
-------------------------------------------------------------------------------
-- File       : souris_ps2.vhd
-- Author     :   <nouel@SARE>
-- Company    : ENSEIRB
-- Last update: 2005/03/11
-- Platform   : 
-------------------------------------------------------------------------------
-- Description: Peripherique pour systeme altera excalibur
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2002/09/24  1.0      nouel	Created
-- Juin 2003 : 1.1 rajour raz donnee si pas de mouvement
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY souris_ps2 IS
  PORT (
    clk            : IN    std_logic;  -- horloge systeme 
    reset          : IN    std_logic;  -- reset general du circuit
    adress         : IN    std_logic_vector(1 DOWNTO 0);
    cs             : IN    std_logic;  -- selection du periph
    rd, wr         : IN    std_logic;  -- lecture /ecriture
    data_IN        : IN    std_logic_vector(31 DOWNTO 0);  -- connexion au
    data_OUT       : OUT   std_logic_vector(31 DOWNTO 0);  -- bus avalon
    irq            : OUT   std_logic;  -- interruption
    ms_data        : INOUT std_logic;   -- bidirectionnel
    ms_clk         : INOUT std_logic);   -- la souris fournie l'horloge 
END souris_ps2;

ARCHITECTURE enseirb OF souris_ps2 IS 

  SIGNAL registre_emission : std_logic_vector(11 DOWNTO 0);
  SIGNAL recevoir, emettre, memoriser : boolean;
  SIGNAL init_tempo, direction_donnee: boolean;
  SIGNAL direction_horloge : boolean; 
  SIGNAL mouvement_h : signed(8 DOWNTO 0);
  SIGNAL mouvement_v : signed(8 DOWNTO 0);
  SIGNAL droit : std_logic;
  SIGNAL gauche : std_logic;
  SIGNAL milieu : std_logic;
  SIGNAL x0, y0 : std_logic;
  
  TYPE t_etat IS (debut, tempo, rts, emission,
                  reception1, debut_trame, reception2,memo);
  SIGNAL etat : t_etat;
  SIGNAL fin_tempo : std_ulogic;
  SIGNAL fin_octet : std_ulogic;
  SIGNAL fin_emission : std_ulogic;
  SIGNAL fin_trame : std_ulogic;
  SIGNAL donnee_e : std_ulogic;
  SIGNAL donnee_s : std_ulogic;
  SIGNAL h_e : std_ulogic;
  SIGNAL hs_filtree : std_ulogic;       -- horloge souris filtree
  SIGNAL registre_etat : std_logic_vector(31 DOWNTO 0);  -- adresse 0
  SIGNAL registre_H    : std_logic_vector(31 DOWNTO 0);  -- adresse 1
  SIGNAL registre_V    : std_logic_vector(31 DOWNTO 0);  -- adresse 2
  SIGNAL aut_irq : std_ulogic;          -- autorisation irq
  
BEGIN  -- enseirb
  
-- separation des donnes en entree et sortie
  donnee_e <=ms_data;
  ms_data <= donnee_s WHEN direction_donnee ELSE 'Z';

  -- separation des horloges en entree et sortie
  ms_clk <= '0' WHEN direction_horloge ELSE 'Z';
  h_e <= ms_clk ;

  -- par securite, on introduit un filtre anti-parasite sur
  -- l'horloge souris qui consiste  valider un 1 ou un 0 par 8 echantillons
  filtre: PROCESS 
    VARIABLE registre : std_logic_vector(7 DOWNTO 0);
  BEGIN  -- PROCESS filtre
    WAIT UNTIL rising_edge(clk);
    registre := registre(6 DOWNTO 0) & h_e ;
    IF registre = "11111111" THEN
      hs_filtree <= '1';                -- filtre de 320 ns
    END IF;
    IF registre = "00000000" THEN
      hs_filtree <= '0'; 
    END IF;
  END PROCESS filtre;

-- la temporisation sert  maintenir l'horloge souris
  -- pendant 100 us environ lorsque le systeme veut commander
  -- la souris
  temporisation: PROCESS 
    CONSTANT duree : natural := 8000;    -- 100 us a 80Mhz
    VARIABLE decompteur : natural RANGE 0 TO duree;
  BEGIN  -- PROCESS temporisation
    WAIT UNTIL rising_edge(clk);  -- synchrone
    IF init_tempo THEN
      decompteur := duree;
      fin_tempo <= '0';
    ELSIF decompteur /=  0 THEN       
      decompteur := decompteur -1;
    ELSE
      fin_tempo <= '1';
    END IF;
  END PROCESS temporisation;

  -- le registre emission ne sert qu'a emettre la commande "stream mode"
  emiss: PROCESS(emettre, hs_filtree)
    VARIABLE nb_bits : natural RANGE 0 TO 11;
    VARIABLE registre_emission : std_logic_vector(11 DOWNTO 0);
    CONSTANT mot_f4 : std_logic_vector(11 DOWNTO 0) := "010111101000";
  BEGIN  -- PROCESS emission
    IF emettre = false THEN                    -- asynchrone
      registre_emission := mot_f4;
      nb_bits := 0;
    ELSIF falling_edge(hs_filtree) THEN  -- synchrone
      registre_emission := '0' & registre_emission(11 DOWNTO 1); 
      nb_bits := nb_bits + 1;
    END IF;
    IF nb_bits = 11 THEN
      fin_emission <= '1';
    ELSE
      fin_emission <= '0';
    END IF;
    donnee_s <= registre_emission(0);
  END PROCESS emiss;

  -- le registre de reception sert soit  recevoir l'acquittement
  -- du mot de commande precedent
  -- soit de recevoir une trame complete
  reception : PROCESS(recevoir, hs_filtree)
    VARIABLE nb_bits : natural RANGE 0 TO 33;
    VARIABLE registre_reception : std_logic_vector(32 DOWNTO 0);
  BEGIN  -- PROCESS emission
      IF recevoir = false  THEN -- asynchrone
      nb_bits := 0;
    ELSIF falling_edge(hs_filtree) THEN  -- synchrone
      registre_reception := donnee_e & registre_reception(32 DOWNTO 1) ;
      nb_bits := nb_bits + 1;
    END IF;
    IF nb_bits = 11 THEN
      fin_octet <= '1';
      -- traitement ignor pour l'instant
    ELSE
      fin_octet <= '0';
    END IF;
    IF nb_bits = 33 THEN
      fin_trame <= '1';
    ELSE
      fin_trame <= '0';
    END IF;
    mouvement_v <= signed(registre_reception(6) &
                          registre_reception(30 DOWNTO 23));
    mouvement_h <= signed(registre_reception(5) &
                          registre_reception(19 DOWNTO 12));
    gauche <= registre_reception(1);
    milieu <= registre_reception(3);
    droit <= registre_reception(2);
    x0 <= registre_reception(7);
    y0 <= registre_reception(8);
  END PROCESS reception;

  trait_irq: PROCESS
  BEGIN  -- PROCESS flag_irq
     WAIT UNTIL rising_edge(clk);  -- synchrone
     IF init_tempo THEN
       aut_irq <= '0';
     ELSIF cs = '1' AND wr = '1' THEN
       aut_irq <= data_IN(0);
     END IF;
  END PROCESS trait_irq;

  -- Les 3 registres
  -- 0: status 
  -- 1: mouvement horizontal signe sur 32 bits
  -- 2: mouvement vertical signe sur 32 bits
  
memor: PROCESS 
BEGIN  -- PROCESS memor
   WAIT UNTIL rising_edge(clk);  -- synchrone  
   IF init_tempo THEN                   -- reset indirect
     irq <= '0';   -- interruption remis  0
     registre_etat <= (OTHERS => '0');
     registre_H <= (OTHERS => '0');
     registre_V <= (OTHERS => '0');
   ELSIF memoriser THEN
     registre_etat <= "000000000000000000000000"
           & '1' & '1'& '1' & y0 & x0 & gauche & milieu & droit;
     registre_h <= std_logic_vector(resize(mouvement_h,32));
     registre_v <= std_logic_vector(resize(mouvement_v,32));
     irq <= aut_irq ;                   -- si fin de trame et autorisation
   END IF;
   IF cs = '1' AND rd = '1' THEN
      irq <= '0';   -- interruption remis  0
      CASE adress IS
        WHEN "00"  =>   registre_etat(7) <= '0';  -- acquitte g m d
        WHEN "01"  =>   registre_etat(6) <= '0';  -- acquitte registe h
        WHEN "10"  =>   registre_etat(5) <= '0';  -- acquitte registre v
        WHEN OTHERS => NULL;
      END CASE;
   END IF;
END PROCESS memor;

  -- le sequenceur enchaine les differentes etapes et boucle
  -- sur la reception des trames et leur traitement
  sequenceur: PROCESS
  BEGIN  -- PROCESS sequenceur
    WAIT UNTIL rising_edge(clk);  -- synchrone
    IF reset = '1' THEN
      etat <= debut;
    ELSE
      CASE etat IS
        WHEN debut => etat <= tempo;
        WHEN tempo => IF fin_tempo = '1' THEN
                        etat <= rts;
                      END IF;
        WHEN rts => etat <= emission;
        WHEN emission => IF fin_emission = '1' THEN
                           etat <= reception1;
                         END IF;
        WHEN reception1 => IF fin_octet = '1'  THEN
                             etat <= debut_trame ;
                           END IF;
        WHEN debut_trame =>  etat <= reception2;
        WHEN reception2 => IF fin_trame = '1' THEN
                             etat <= memo;
                           END IF;
        WHEN memo =>  etat <= reception2;
      END CASE;
    END IF;
  END PROCESS sequenceur;

  init_tempo <= (etat = debut);

  direction_horloge <= (etat = tempo);

  emettre <= (etat = emission);

  direction_donnee <= (etat = emission) OR (etat = rts) ;

  recevoir <= (etat = reception1) OR (etat = reception2);

  memoriser <= (etat = memo);

  -- selection des registres sur le bus de sortie
  data_OUT <= registre_etat WHEN (adress = "00" AND cs = '1' AND rd = '1')
              ELSE registre_h WHEN (adress = "01" AND cs = '1' AND rd = '1')
              ELSE registre_v WHEN (adress = "10" AND cs = '1' AND rd = '1')
              ELSE (OTHERS => '1');
END enseirb; 
